#include "Control.h"
#include "Config.h"

Control::Control() {
}

void Control::setPin(uint8_t p) {
  pin =p; 
}

void AnalogControl::setPin(uint8_t p) {
  pin =p; 
#ifdef STM32_BOARD  
  pinMode(pin,INPUT_ANALOG);
#endif
} 

void AnalogControl::handle() {

  uint16_t v = analogRead(pin);
 uint16_t c = ch->getCenterAnalog();
 if (v<c) {
  v = map(v, 0, c, 1000, 1500);
 } else {
  v = map(v, c, ANA_MAX, 1500, 2000);
 }
  ch->setRawValue(v);
}



void AnalogControl::resetCenterAnalog() {
  uint16_t v = analogRead(pin);
  ch->setCenterAnalog(v);
}

boolean AnalogControl::resetPercent() {
  uint16_t v = analogRead(pin);
 uint16_t c = ch->getCenterAnalog();
 uint8_t p = 100;
 if (v<c) {
   p = (c-v)*100/c;
 } else {
   p = (v-c)*100/(ANA_MAX-c);
 }

 if ( p >= 20) {
    ch->setPercent(p);
    return true;
 }
 return false;
}

void BtnControl::setPin(uint8_t p) {
  pin =p; 
  pinMode(pin,INPUT_PULLUP);
} 

boolean BtnControl::isTriggered() {
  
  if ( digitalRead(pin) == LOW) {
    delayMicroseconds(1);
    return 1;
  }
  return  0;
}


void FuncBtnControl::handle() {
  if (isTriggered()) {
    func();
  }
}

OffsetBtnControl::OffsetBtnControl() {
  dec = false;
}

boolean OffsetBtnControl::isTriggered() {
  static long last = 0;
  long now = millis(); 

  if ( now - last > 150 && digitalRead(pin) == LOW) {
    delayMicroseconds(1);
    last = now;
    return 1;
  }
  return  0;
}

void OffsetBtnControl::handle() {
  if (isTriggered()) {
    ch->changeOffset(dec);
  }
}

void ReverseBtnControl::handle() {
  if (isTriggered()) {
    ch->setReversed(true);
  }  else {
    ch->setReversed(false);
  }
}

//===============================
#ifdef BAT_ALART  

#define BAT_FACTOR  2
#define BAT_BASE  3.3
#define BAT_MIN  3.6

void BatAlertControl::handle() {
  static long last = 0;
  long now = millis(); 
  if ( now - last > 1000*10) {  //every 10 seconds check
       float v = (float)(analogRead(BAT_PIN) * BAT_FACTOR)/ANA_MAX * BAT_BASE;
       if (v < BAT_MIN)  { // 1.1 factor only for 1S
      tone(Tone_pin,1600,800);
      }
      last = now;      
  }
}
#endif  


#ifdef TEST_CHAN
TestControl::TestControl() {
}

void TestControl::handle() {
  static long last = 0;
  uint16_t v;
  long now = millis(); 
  if ( now - last > 10) {
    v = ch->getRawValue() + 20;
    if (v >= PWM_MAX) {
       v = PWM_MIN;
    }
    ch->setRawValue(v);
    last = now;
  }
}
#endif


